/************************************************************************//**
 *
 * @file    wdt_reg.h
 *
 * @brief   Definition of the Watchdog Timer(WDT) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef WDT_REG_H__
#define WDT_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <c17_mcu_select.h>

/// WDT Clock Control Register
typedef union WDTCLK_REG_TAG {
    unsigned short  reg;
    struct WDTCLK_REG_BIT_TAG {
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
    defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
    defined(C17_MCUSEL_W15) || \
    defined(C17_MCUSEL_W18) || \
    defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)
        /**
         * These bits select the clock source of WDT.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+---------------------------------------+
         *  |             |              CLKSRC[1:0]              |
         *  |             +----------+--------+----------+--------+
         *  |             |    0x0   |   0x1  |    0x2   |  0x3   |
         *  | CLKDIV[1:0] +----------+--------+----------+--------+
         *  |             |          |        |          |        |
         *  |             |   IOSC   |  OSC1  |   OSC3   | EXOSC  |
         *  |             |          |        |          |        |
         *  +-------------+----------+--------+----------+--------+
         *  |     0x3     | 1/16,384 |  1/128 | 1/16,384 |  1/1   |
         *  +-------------+----------+        +----------+        +
         *  |     0x2     |  1/8,192 |        |  1/8,192 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x1     |  1/4,096 |        |  1/4,096 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x0     |  1/2,048 |        |  1/2,048 |        |
         *  +-------------+----------+--------+----------+--------+
         */
#endif
#if defined(C17_MCUSEL_M01) || \
    defined(C17_MCUSEL_589)
        /**
         * These bits select the clock source of WDT.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+---------------------------------------+
         *  |             |              CLKSRC[1:0]              |
         *  |             +----------+--------+----------+--------+
         *  |             |    0x0   |   0x1  |    0x2   |  0x3   |
         *  | CLKDIV[1:0] +----------+--------+----------+--------+
         *  |             |          |        |          |        |
         *  |             |   IOSC   |  OSC1  |   OSC3   | EXOSC  |
         *  |             |          |        |          |        |
         *  +-------------+----------+--------+----------+--------+
         *  |     0x3     | 1/65,536 |  1/128 | 1/65,536 |  1/1   |
         *  +-------------+----------+        +----------+        +
         *  |     0x2     | 1/32,768 |        | 1/32,768 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x1     | 1/16,384 |        | 1/16,384 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x0     |  1/8,192 |        |  1/8,192 |        |
         *  +-------------+----------+--------+----------+--------+
         */
#endif
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
    defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
    defined(C17_MCUSEL_W15) || \
    defined(C17_MCUSEL_W18) || \
    defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)
        /**
         * These bits select the division ratio of the WDT operating clock.
         *  should be set to around 256 Hz.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+---------------------------------------+
         *  |             |              CLKSRC[1:0]              |
         *  |             +----------+--------+----------+--------+
         *  |             |    0x0   |   0x1  |    0x2   |  0x3   |
         *  | CLKDIV[1:0] +----------+--------+----------+--------+
         *  |             |          |        |          |        |
         *  |             |   IOSC   |  OSC1  |   OSC3   | EXOSC  |
         *  |             |          |        |          |        |
         *  +-------------+----------+--------+----------+--------+
         *  |     0x3     | 1/16,384 |  1/128 | 1/16,384 |  1/1   |
         *  +-------------+----------+        +----------+        +
         *  |     0x2     |  1/8,192 |        |  1/8,192 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x1     |  1/4,096 |        |  1/4,096 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x0     |  1/2,048 |        |  1/2,048 |        |
         *  +-------------+----------+--------+----------+--------+
         */
#endif
#if defined(C17_MCUSEL_M01) || \
    defined(C17_MCUSEL_589)
        /**
         * These bits select the division ratio of the WDT operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+---------------------------------------+
         *  |             |              CLKSRC[1:0]              |
         *  |             +----------+--------+----------+--------+
         *  |             |    0x0   |   0x1  |    0x2   |  0x3   |
         *  | CLKDIV[1:0] +----------+--------+----------+--------+
         *  |             |          |        |          |        |
         *  |             |   IOSC   |  OSC1  |   OSC3   | EXOSC  |
         *  |             |          |        |          |        |
         *  +-------------+----------+--------+----------+--------+
         *  |     0x3     | 1/65,536 |  1/128 | 1/65,536 |  1/1   |
         *  +-------------+----------+        +----------+        +
         *  |     0x2     | 1/32,768 |        | 1/32,768 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x1     | 1/16,384 |        | 1/16,384 |        |
         *  +-------------+----------+        +----------+        +
         *  |     0x0     |  1/8,192 |        |  1/8,192 |        |
         *  +-------------+----------+--------+----------+--------+
         */
#endif
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * This bit sets whether the WDT operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the WDT operating clock is supplied in DEBUG mode or not.
         *    1 (R/WP): Clock supplied in DEBUG mode
         *    0 (R/WP): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} WDTCLK_REG;

/// WDT Control Register
typedef union WDTCTL_REG_TAG {
    unsigned short  reg;
    struct WDTCTL_REG_BIT_TAG {
        /**
         * These bits control WDT to run and stop.
         *  .WDTRUN[3:0]
         *  These bits control WDT to run and stop.
         *    0xa (R/WP): Stop
         *    Values other than 0xa (R/WP): Run
         */
        unsigned short      WDTRUN      : 4;
        /**
         * This bit resets WDT.
         * .WDTCNTRST
         *  This bit resets WDT.
         *    1 (WP): Reset
         *    0 (WP): Ignored
         *    0 (R): Always 0 when being read
         */
        unsigned short      WDTCNTRST   : 1;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * This bit indicates that a counter overflow and NMI have occurred.
         * .STATNMI
         *  This bit indicates that a counter compare match and NMI have occurred.
         *    1 (R): NMI (counter compare match) occurred
         *    0 (R): NMI not occurred
         */
        unsigned short      STATNMI     : 1;
        /**
         * .MIXRST
         *  This bit sets the WDT operating mode.
         *   1 (R/WP): NMI mode
         *   0 (R/WP): Reset mode
         */
        unsigned short      NMIXRST     : 1;
        unsigned short      reserved_2  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} WDTCTL_REG;

#ifdef __cplusplus
}
#endif

#endif // WDT_REG_H__
