/************************************************************************//**
 *
 * @file    c17_snda.h
 *
 * @brief   This is SNDA driver header file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#ifndef C17_SNDA_H__
#define C17_SNDA_H__

#include <c17_mcu_select.h>

/* --- definition --- */
enum
{
	C17_SND_CLKSRC_IOSC,				///< Clock Source = IOSC
	C17_SND_CLKSRC_OSC1,				///< Clock Source = OSC1
	C17_SND_CLKSRC_OSC3,				///< Clock Source = OSC3
	C17_SND_CLKSRC_EXT					///< Clock Source = External OSC
};

#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
enum
{
	C17_SND_CLKDIV_16,					///< Clock Division = 1/16
	C17_SND_CLKDIV_32,					///< Clock Division = 1/32
	C17_SND_CLKDIV_64,					///< Clock Division = 1/64
	C17_SND_CLKDIV_128,					///< Clock Division = 1/128
	C17_SND_CLKDIV_256,					///< Clock Division = 1/256
	C17_SND_CLKDIV_512					///< Clock Division = 1/512
};
#else
enum
{
	C17_SND_CLKDIV_4,					///< Clock Division = 1/4
	C17_SND_CLKDIV_8,					///< Clock Division = 1/8
	C17_SND_CLKDIV_16,					///< Clock Division = 1/16
	C17_SND_CLKDIV_32,					///< Clock Division = 1/32
	C17_SND_CLKDIV_64,					///< Clock Division = 1/64
	C17_SND_CLKDIV_128					///< Clock Division = 1/128
};
#endif

enum
{
	C17_SND_CLKDIV_1,					///< Clock Division = 1/1
};

enum
{
	C17_SND_MODE_NORMAL,				///< Sound mode - Normal.
	C17_SND_MODE_ONE_SHOT,				///< Sound mode - One shot.
	C17_SND_MODE_MELODY,				///< Sound mode - Melody.
};

enum
{
	C17_SND_DURATION_0,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_1,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_2,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_3,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_4,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_5,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_6,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_7,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_8,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_9,					///< one-shot duration(one-shot mode).
	C17_SND_DURATION_10,				///< one-shot duration(one-shot mode).
	C17_SND_DURATION_11,				///< one-shot duration(one-shot mode).
	C17_SND_DURATION_12,				///< one-shot duration(one-shot mode).
	C17_SND_DURATION_13,				///< one-shot duration(one-shot mode).
	C17_SND_DURATION_14,				///< one-shot duration(one-shot mode).
	C17_SND_DURATION_15,				///< one-shot duration(one-shot mode).
};

enum
{
	C17_SND_TEMPO_0,					///< tempo(melody mode).
	C17_SND_TEMPO_1,					///< tempo(melody mode).
	C17_SND_TEMPO_2,					///< tempo(melody mode).
	C17_SND_TEMPO_3,					///< tempo(melody mode).
	C17_SND_TEMPO_4,					///< tempo(melody mode).
	C17_SND_TEMPO_5,					///< tempo(melody mode).
	C17_SND_TEMPO_6,					///< tempo(melody mode).
	C17_SND_TEMPO_7,					///< tempo(melody mode).
	C17_SND_TEMPO_8,					///< tempo(melody mode).
	C17_SND_TEMPO_9,					///< tempo(melody mode).
	C17_SND_TEMPO_10,					///< tempo(melody mode).
	C17_SND_TEMPO_11,					///< tempo(melody mode).
	C17_SND_TEMPO_12,					///< tempo(melody mode).
	C17_SND_TEMPO_13,					///< tempo(melody mode).
	C17_SND_TEMPO_14,					///< tempo(melody mode).
	C17_SND_TEMPO_15,					///< tempo(melody mode).
};

enum
{
	C17_SND_TIE_DISABLE,				///< specifies a tie disable.
	C17_SND_TIE_ENABLE,					///< specifies a tie enable.
};

enum
{
	C17_SND_TYPE_NOTE,					///< output type in note.
	C17_SND_TYPE_REST,					///< output type in rest.
};

//  Melody Status.
#define SNDA_MELODY_STOP		(0)
#define SNDA_MELODY_RUN			(1)

// Sound length NOTE,REST.
#define C17_SND_LEN_32			(0x00)	///< Thirty-second note.
#define C17_SND_LEN_16			(0x01)	///< Sixteenth note
#define C17_SND_LEN_8			(0x03)	///< Eighth note
#define C17_SND_LEN_DOTTED_8	(0x05)	///< Dotted eighth note
#define C17_SND_LEN_4			(0x07)	///< Quarter note
#define C17_SND_LEN_DOTTED_4	(0x0b)	///< Dotted quarter note
#define C17_SND_LEN_2			(0x0f)	///< Half note

// Sound scale Data.(S = #)
#define C17_SND_SCALE_C3		(0xf8)
#define C17_SND_SCALE_CS3		(0xea)
#define C17_SND_SCALE_D3		(0xdd)
#define C17_SND_SCALE_DS3		(0xd1)
#define C17_SND_SCALE_E3		(0xc5)
#define C17_SND_SCALE_F3		(0xba)
#define C17_SND_SCALE_FS3		(0xaf)
#define C17_SND_SCALE_G3		(0xa5)
#define C17_SND_SCALE_GS3		(0x9c)
#define C17_SND_SCALE_A3		(0x93)
#define C17_SND_SCALE_AS3		(0x8b)
#define C17_SND_SCALE_B3		(0x83)
#define C17_SND_SCALE_C4		(0x7c)
#define C17_SND_SCALE_CS4		(0x75)
#define C17_SND_SCALE_D4		(0x6e)
#define C17_SND_SCALE_DS4		(0x68)
#define C17_SND_SCALE_E4		(0x62)
#define C17_SND_SCALE_F4		(0x5c)
#define C17_SND_SCALE_FS4		(0x57)
#define C17_SND_SCALE_G4		(0x52)
#define C17_SND_SCALE_GS4		(0x4e)
#define C17_SND_SCALE_A4		(0x49)
#define C17_SND_SCALE_AS4		(0x45)
#define C17_SND_SCALE_B4		(0x41)
#define C17_SND_SCALE_C5		(0x3d)
#define C17_SND_SCALE_CS5		(0x3a)
#define C17_SND_SCALE_D5		(0x37)
#define C17_SND_SCALE_DS5		(0x33)
#define C17_SND_SCALE_E5		(0x30)
#define C17_SND_SCALE_F5		(0x2e)
#define C17_SND_SCALE_FS5		(0x2b)
#define C17_SND_SCALE_G5		(0x29)
#define C17_SND_SCALE_GS5		(0x26)
#define C17_SND_SCALE_A5		(0x24)
#define C17_SND_SCALE_AS5		(0x22)
#define C17_SND_SCALE_B5		(0x20)
#define C17_SND_SCALE_C6		(0x1e)

#define C17_SND_LAST_DATA		(0)		///< Last data when setting melody data.

/* --- external declaration --- */
extern void c17intSnda(void)	__attribute__ ((interrupt_handler));

extern void c17initSnda(void);
extern void c17setSnda(unsigned short sndClkSrc, unsigned short sndClkDiv);
extern void c17startSnda(void);
extern void c17stopSnda(void);
extern unsigned short c17getSndaMode(void);
extern void c17stopSndaSound(void);
extern void c17runSndaNormal(float duty, float freq);
extern void c17runSndaOneShot(float duty, float freq, unsigned short duration);
extern void c17setMelodyNote(unsigned short duration, unsigned short scale, unsigned short tie, unsigned short melody_data[][4]);
extern void c17setMelodyRest(unsigned short duration, unsigned short melody_data[][4]);
extern void c17runSndaMelody(unsigned short melody_data[][4], unsigned short tempo);
extern int c17getSndaMelodyState(void);
extern void c17clearSndaMelodyData(unsigned short size, unsigned short melody_data[][4]);

#endif	// C17_SNDA_H__

