/************************************************************************//**
 *
 * @file    c17_spia.h
 *
 * @brief   This is SPIA driver header file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#ifndef C17_SPIA_H__
#define C17_SPIA_H__

#include <c17_mcu_select.h>

/* --- definition --- */
enum
{
	C17_SPIA_MODE_SLAVE,			///< Slave mode.
	C17_SPIA_MODE_MASTER			///< Master mode.
};

enum
{
	C17_SPIA_PULL_DISABLE,			///< SPIA input pins pull-up or pull-down function disable.
	C17_SPIA_PULL_ENABLE,			///< SPIA input pins pull-up or pull-down function enable.
};

enum
{
	C17_SPIA_MODE0,					///< SPI Clock Phase and Polarity. CPOL=0, CPHA=0.
	C17_SPIA_MODE1,					///< SPI Clock Phase and Polarity. CPOL=0, CPHA=1.
	C17_SPIA_MODE2,					///< SPI Clock Phase and Polarity. CPOL=1, CPHA=0.
	C17_SPIA_MODE3,					///< SPI Clock Phase and Polarity. CPOL=1, CPHA=1.
};

enum
{
	C17_SPIA_MSB,					///< Select MSB first
	C17_SPIA_LSB,					///< Select LSB first
};

enum
{
	C17_SPIA_DATA_BIT2 = 1,			///< Length of transfer data = 2bit.
	C17_SPIA_DATA_BIT3,				///< Length of transfer data = 3bit.
	C17_SPIA_DATA_BIT4,				///< Length of transfer data = 4bit.
	C17_SPIA_DATA_BIT5,				///< Length of transfer data = 5bit.
	C17_SPIA_DATA_BIT6,				///< Length of transfer data = 6it.
	C17_SPIA_DATA_BIT7,				///< Length of transfer data = 7bit.
	C17_SPIA_DATA_BIT8,				///< Length of transfer data = 8bit.
	C17_SPIA_DATA_BIT9,				///< Length of transfer data = 9bit.
	C17_SPIA_DATA_BIT10,			///< Length of transfer data = 10bit.
	C17_SPIA_DATA_BIT11,			///< Length of transfer data = 11bit.
	C17_SPIA_DATA_BIT12,			///< Length of transfer data = 12bit.
	C17_SPIA_DATA_BIT13,			///< Length of transfer data = 13bit.
	C17_SPIA_DATA_BIT14,			///< Length of transfer data = 14bit.
	C17_SPIA_DATA_BIT15,			///< Length of transfer data = 15bit.
	C17_SPIA_DATA_BIT16,			///< Length of transfer data = 16bit.
};

/* --- external declaration --- */
extern void c17initSpia(void);
extern void c17startSpia(unsigned short spiChannel);
extern void c17stopSpia(unsigned short spiChannel);
extern void c17setSpia(unsigned short spiChannel, unsigned short spiMode, unsigned short spiPullupdown, unsigned short spiProtocol, unsigned short spiFormat, unsigned short spiBit);
extern void c17assertSpiaSs(unsigned short spiChannel);
extern void c17negateSpiaSs(unsigned short spiChannel);
extern void c17sendSpiaData(unsigned short spiChannel, char *spiData, unsigned short spiSize);
extern void c17rcvSpiaData(unsigned short spiChannel, char *spiData, unsigned short spiSize);

#endif	// SPIA_H__

