/*
 * spia_asm.s
 */


.global c17sendSPIDataAsm
.global c17rcvSpiaDataAsm

// SPI data send.
// format: unsigned short c17sendSPIDataAsm(unsigned short spiChannel, unsigned char *spiData, unsigned short spiSize);
// param: unsigned short spiChannel SPI channel number.
// param: unsigned char *spiData    send data
// param: unsigned short spiSize    send data length
// return: unsigned char            TRUE
c17sendSPIDataAsm:
    ld.a    -[%sp], %r4
    ld.a    -[%sp], %r5
    ld.a    -[%sp], %r6


    cmp		%r0, 0x01
    jreq    _spi_send_50
       						// CH.0
    sld     %r4, 0x43b8    	//%r4:SPI0INTF
    sld     %r5, 0x43b4     //%r5:SPI0TXD
    jpr		_spi_send_100

_spi_send_50:
							//CH.1
	sld     %r4, 0x5278     //%r4:SPI1INTF
    sld     %r5, 0x5274     //%r5:SPI1TXD


_spi_send_100:
    ld.b    %r6, [%r4]      //Check TBEIF
    and     %r6, 0x01
    jreq    _spi_send_100

    ld.b    %r6, [%r1]+		//Get send data
    ld      [%r5], %r6		//SPINTXD
    sub     %r2, 1
    jrne    _spi_send_100

_spi_send_300:
   	ld.b    %r6, [%r4]      //Check BSY
    xand    %r6, 0x80
    xcmp	%r6, 0x80
    jreq    _spi_send_300

	ld    	%r0,1
    ld.a    %r6, [%sp]+
    ld.a    %r5, [%sp]+
    ld.a    %r4, [%sp]+
    ret



// SPI data receive.
// format: unsigned short c17rcvSpiaDataAsm(unsigned short spiChannel, unsigned char *spiData, unsigned short spiSize);
// param: unsigned short spiChannel SPI channel number.
// param: unsigned char *spiData    receive data
// param: unsigned short spiSize    receive data length
// return: unsigned char            TRUE

c17rcvSpiaDataAsm:
    ld.a    -[%sp], %r4
    ld.a    -[%sp], %r5
    ld.a    -[%sp], %r6
    ld.a    -[%sp], %r7


    cmp		%r0, 0x01
    jreq    _spi_receive_50
    sld     %r3, 0x43b0     //%r4:SPI0MOD
    sld     %r4, 0x43b8     //%r4:SPI0INTF
    sld     %r5, 0x43b4     //%r5:SPI0TXD
    sld     %r6, 0x43b6     //%r6:SPI0RXD
    jpr		_spi_receive_80

_spi_receive_50:
	sld     %r3, 0x5270     //%r3:SPI1MOD
    sld     %r4, 0x5278     //%r4:SPI1INTF
    sld     %r5, 0x5274     //%r5:SPI1TXD
    sld     %r6, 0x5276     //%r6:SPI1RXD

_spi_receive_80:
    ld      %r7, [%r6]		//Dummy Read

    ld      %r3, [%r3]		//SPINMOD
    and		%r3, 0x01

_spi_receive_100:
	cmp		%r3, 0x01		//Master mode
	jrne    _spi_receive_200

    ld.b    %r7, [%r4]      //Check TBEIF
    and     %r7, 0x01
    jreq    _spi_receive_100

    xld    	%r7, 0xaa		//dummy send data
    ld      [%r5], %r7		//SPI0TXD

_spi_receive_200:
    ld.b    %r7, [%r4]      //Check RBFIF
    and     %r7, 0x02
    jreq    _spi_receive_200

    ld      %r7, [%r6]
    ld.b    [%r1]+, %r7
    sub     %r2, 1
    jrne    _spi_receive_100

_spi_receive_300:
   	ld.b    %r7, [%r4]      //Check BSY
    xand    %r7, 0x80
    xcmp	%r7, 0x80
    jreq    _spi_receive_300


	ld    	%r0,1
    ld.a    %r7, [%sp]+
    ld.a    %r6, [%sp]+
    ld.a    %r5, [%sp]+
    ld.a    %r4, [%sp]+
    ret






