/****************************************************************************
 *
 * @file	t16b.h
 * @brief	This is T16 PWM Timer(T16B) header file.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *          - 01 Sep. 2016    1.00
 *				- First revision.
 *
 ***************************************************************************/
#ifndef T16B_H__
#define T16B_H__

#include <c17_mcu_select.h>

/* --- definition --- */
enum
{
	C17_T16B_CLKSRC_IOSC,				///< Clock Source = IOSC
	C17_T16B_CLKSRC_OSC1,				///< Clock Source = OSC1
	C17_T16B_CLKSRC_OSC3,				///< Clock Source = OSC3
	C17_T16B_CLKSRC_EXT,				///< Clock Source = External OSC
	C17_T16B_CLKSRC_EXCL0,				///< Clock Source = External CLn0
	C17_T16B_CLKSRC_EXCL1,				///< Clock Source = External CLn1
	C17_T16B_CLKSRC_EXCL0INV,			///< Clock Source = External CLn0 inverted input
	C17_T16B_CLKSRC_EXCL1INV			///< Clock Source = External CLn1 inverted input
};

enum
{
	C17_T16B_CLKDIV_1,					///< Clock Division = 1/1
	C17_T16B_CLKDIV_2,					///< Clock Division = 1/2
	C17_T16B_CLKDIV_4,					///< Clock Division = 1/4
	C17_T16B_CLKDIV_8,					///< Clock Division = 1/8
	C17_T16B_CLKDIV_16,					///< Clock Division = 1/16
	C17_T16B_CLKDIV_32,					///< Clock Division = 1/32
	C17_T16B_CLKDIV_64,					///< Clock Division = 1/64
	C17_T16B_CLKDIV_128,				///< Clock Division = 1/128
	C17_T16B_CLKDIV_256,				///< Clock Division = 1/256
	C17_T16B_CLKDIV_512,				///< Clock Division = 1/512
	C17_T16B_CLKDIV_1024,				///< Clock Division = 1/1024
	C17_T16B_CLKDIV_2048,				///< Clock Division = 1/2048
	C17_T16B_CLKDIV_4096,				///< Clock Division = 1/4096
	C17_T16B_CLKDIV_8192,				///< Clock Division = 1/8192
	C17_T16B_CLKDIV_16384,				///< Clock Division = 1/16384
	C17_T16B_CLKDIV_32768				///< Clock Division = 1/32768
};

enum
{
	C17_T16B_UP,						///< Up count mode
	C17_T16B_DOWN,						///< Down count mode
	C17_T16B_UP_DOWN					///< Up/Down count mode
};

enum
{
	C17_T16B_REPEAT,					///< Repeat mode.
	C17_T16B_ONESHOT					///< One shot mode.
};

/// Compare interrupt counter data max.
#define T16B_CH0_INT_CAP_DATA_MAX	(20)


/* --- external declaration --- */
extern void c17intT16bCh0(void)	__attribute__ ((interrupt_handler));

extern void c17initT16b(void);
extern void c17setT16b(unsigned short t16BChannel, unsigned short t16BClkSrc, unsigned short t16BClkDiv, unsigned short t16BUpDown, unsigned short t16BMode);
extern void c17startT16b(unsigned short t16BChannel);
extern void c17stopT16b(unsigned short t16BChannel);
extern unsigned short c17getT16bCh0ComparatorCount(void);
extern void c17setT16bCh0ComparatorCount(unsigned short count);
extern unsigned short c17getT16bCh0TimerCount(void);
extern unsigned short c17getT16bCh0MaxCount(void);
extern void c17setT16bCh0MaxCount(unsigned short count);
extern void c17resetT16bCh0IntCount(void);
extern unsigned short c17getT16bCh0IntMaxCount(void);
extern unsigned short c17getT16bCh0IntZeroCount(void);
extern unsigned short c17getT16bCh0IntCaptureCount(void);
extern unsigned short c17getT16bCh0IntCaptureCountData(unsigned int index);
extern unsigned short c17getT16bCh0IntComparatorCount(void);

#endif	// T16B_H__

