/************************************************************************//**
 *
 * @file    flashc_reg.h
 *
 * @brief   Definition of the Flash Controller(FLASHC) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef FLASHC_REG_H__
#define FLASHC_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <c17_mcu_select.h>

/// FLASHC Flash Read Cycle Register
typedef union FLASHCWAIT_REG_TAG {
    unsigned short  reg;
    struct FLASHCWAIT_REG_BIT_TAG {
#if  defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
       /**
         * These bits set the number of bus access cycles for reading from the Flash memory.
         * .RDWAIT[1:0]
         *  These bits set the number of bus access cycles for reading from the Flash memory.
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |     21.0MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |     18.9MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |     12.6MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |      6.3MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         */
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
        /**
         * These bits set the number of bus access cycles for reading from the Flash memory.
         * .RDWAIT[1:0]
         *  These bits set the number of bus access cycles for reading from the Flash memory.
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |     17.12MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |     17.12MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |      12.6MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |       6.3MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         */
#endif
#if defined(C17_MCUSEL_M10) || \
    defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
    defined(C17_MCUSEL_M40) || \
    defined(C17_MCUSEL_589)
        /**
         * These bits set the number of bus access cycles for reading from the Flash memory.
         * .RDWAIT[1:0]
         *  These bits set the number of bus access cycles for reading from the Flash memory.
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |      16.8MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |      16.8MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |      12.6MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |       6.3MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         */
#endif
#if defined(C17_MCUSEL_F63)
        /**
         * These bits set the number of bus access cycles for reading from the Flash memory.
         * .RDWAIT[1:0]
         *  These bits set the number of bus access cycles for reading from the Flash memory.
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |      16.0MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |      16.0MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |      12.6MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |       6.3MHz(Max.)     |
         * +-------------+-----------------------------+------------------------+
         */
#endif
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
    defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
    defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
    defined(C17_MCUSEL_W15) || \
    defined(C17_MCUSEL_W18) || \
    defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
    defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
        /**
         * These bits set the number of bus access cycles for reading from the Flash memory.
         * .RDWAIT[1:0]
         *  These bits set the number of bus access cycles for reading from the Flash memory.
         * When VDD = 1.2 to 1.6 V
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |      1.1MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |      1.1MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |      1.1MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |      800kHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         *
         * When VDD = 1.6 to 3.6 V
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |      4.2MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |      4.2MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |      4.2MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |      2.1MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         */
#endif
#if defined(C17_MCUSEL_M01)
        /**
         * These bits set the number of bus access cycles for reading from the Flash memory.
         * .RDWAIT[1:0]
         *  These bits set the number of bus access cycles for reading from the Flash memory.
         * +-------------+-----------------------------+------------------------+
         * | RDWAIT[1:0] | Number of bus Access cycles | System clock frequency |
         * +-------------+-----------------------------+------------------------+
         * |     0x3     |             4               |     16.3MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x2     |             3               |     16.3MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x1     |             2               |     16.3MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         * |     0x0     |             1               |      8.2MHz(Max.)      |
         * +-------------+-----------------------------+------------------------+
         */
#endif
        unsigned short      RDWAIT      : 2;
        unsigned short      reserved_1  : 5;    ///< Reserved.
        /// This bit indicates whether the Flash memory can be accessed or not.
        unsigned short      XBUSY       : 1;
        unsigned short      reserved_2  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} FLASHCWAIT_REG;

#ifdef __cplusplus
}
#endif

#endif // FLASHC_REG_H__
