/************************************************************************//**
 *
 * @file    c17_lcd8b.h
 *
 * @brief   This is LCD8B driver header file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#ifndef C17_LCD8B_H__
#define C17_LCD8B_H__

#include <c17_mcu_select.h>

/* --- definition --- */
enum
{
	C17_LCD_CLKSRC_IOSC,				///< Clock Source = IOSC
	C17_LCD_CLKSRC_OSC1,				///< Clock Source = OSC1
	C17_LCD_CLKSRC_OSC3,				///< Clock Source = OSC3
	C17_LCD_CLKSRC_EXT					///< Clock Source = External OSC
};

enum
{
	C17_LCD_CLKDIV_1					///< Clock Division = 1/1
};

enum
{
	C17_LCD_CLKDIV_OSC3_4,			///< Clock Division = 1/4
	C17_LCD_CLKDIV_OSC3_8,				///< Clock Division = 1/8
	C17_LCD_CLKDIV_OSC3_16,				///< Clock Division = 1/16
	C17_LCD_CLKDIV_OSC3_32,				///< Clock Division = 1/32
	C17_LCD_CLKDIV_OSC3_64,				///< Clock Division = 1/64
	C17_LCD_CLKDIV_OSC3_128			///< Clock Division = 1/128
};

enum
{
	C17_LCD_CLKDIV_IOSC_4,				///< Clock Division = 1/4
	C17_LCD_CLKDIV_IOSC_8,				///< Clock Division = 1/8
	C17_LCD_CLKDIV_IOSC_16,				///< Clock Division = 1/16
	C17_LCD_CLKDIV_IOSC_32,				///< Clock Division = 1/32
	C17_LCD_CLKDIV_IOSC_64,				///< Clock Division = 1/64
	C17_LCD_CLKDIV_IOSC_128				///< Clock Division = 1/128
};

enum
{
	C17_LCD_STATE_OFF,					///< Mode : Display off.
	C17_LCD_STATE_NORMAL,				///< Mode : Normal display.
	C17_LCD_STATE_ALL_ON,				///< Mode : All on.
	C17_LCD_STATE_ALL_OFF				///< Mode : All off.
};

enum
{
	C17_LCD_DISPLAY_AREA0,				///< Area : Display Area 0.
	C17_LCD_DISPLAY_AREA1				///< Area : Display Area 1.
};

enum
{
	C17_LCD_DISPLAY_INVERTED,			///< Inverted display
	C17_LCD_DISPLAY_NORMAL				///< Normal display
};

// Display Memory Address.
#define	LCD8B_SEGRAM_TOP_ADDR		(0x7000)			///< The display data RAM address.
#define	LCD8B_SEGRAM_TOP_ADDR_AREA0	(0x7000) 			///< The display data RAM address(Area0).
#define	LCD8B_SEGRAM_TOP_ADDR_AREA1	(0x7040) 			///< The display data RAM address(Area1).
#if defined(C17_MCUSEL_W14)
#define	LCD8B_SEGRAM_SIZE		( 0x76 )						///< The display data RAM size.
#endif
#if defined(C17_MCUSEL_W15)
#define	LCD8B_SEGRAM_SIZE		( 0x62 )						///< The display data RAM size.
#endif
#if defined(C17_MCUSEL_W16)
#define	LCD8B_SEGRAM_SIZE		( 0x7C )						///< The display data RAM size.
#endif
#if defined(C17_MCUSEL_W18)
#define	LCD8B_SEGRAM_SIZE		( 0x70 )						///< The display data RAM size.
#endif

/* --- external declaration --- */
extern void c17intLcd8b(void)	__attribute__ ((interrupt_handler));

extern void c17initLcd8b(void);
extern void c17setLcd8b(unsigned short lcdClkSrc, unsigned short lcdClkDiv);
extern void c17enableIntLcd8b(void);
extern void c17disableIntLcd8b(void);
extern void c17onLcd8bDisplay(void);
extern void c17offLcd8bDisplay(void);
extern void c17setLcd8bDisplayState(unsigned char state);
extern unsigned char c17getLcd8bDisplayState(void);
extern void c17setLcd8bDisplayInverted(unsigned char display);
extern unsigned char c17getLcd8bDisplayInverted(void);
extern void c17setLcd8bDisplayArea(unsigned char area);
extern void c17setLcd8bCheckerboardPattern(void);

#endif // C17_LCD8B_H__

