/************************************************************************//**
 *
 * @file    c17_svd.h
 *
 * @brief   This is SVD header file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2017. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Modify header file.
 *
 ***************************************************************************/
#ifndef C17_SVD_H__
#define C17_SVD_H__

#include <c17_mcu_select.h>

/* --- definition --- */
enum
{
	C17_SVD_IOSC,				///< SVD  Source = IOSC
	C17_SVD_OSC1,				///< SVD  Source = OSC1
	C17_SVD_OSC3,				///< SVD  Source = OSC3
	C17_SVD_EXT					///< SVD  Source = External
};

enum
{
	C17_SVD_CLKDIV_4,			///< SVD clock division = 1/4
	C17_SVD_CLKDIV_8,			///< SVD clock division = 1/8
	C17_SVD_CLKDIV_16,			///< SVD clock division = 1/16
	C17_SVD_CLKDIV_32,			///< SVD clock division = 1/32
	C17_SVD_CLKDIV_64,			///< SVD clock division = 1/64
	C17_SVD_CLKDIV_128,			///< SVD clock division = 1/128
};

enum
{
	C17_SVD_SUPPLY_HIGH,		///< Power supply voltage(VDD or EXSVD) >=  comparison voltage.
	C17_SVD_SUPPLY_LOW			///< Power supply voltage(VDD or EXSVD) <  comparison voltage.
};

// Comparison voltage for detecting low voltage
enum
{
	C17_SVDC_1200MV = 1,		///< Comparison voltage.
	C17_SVDC_1250MV,			///< Comparison voltage.
	C17_SVDC_1300MV,			///< Comparison voltage.
	C17_SVDC_1350MV,			///< Comparison voltage.
	C17_SVDC_1400MV,			///< Comparison voltage.
	C17_SVDC_1450MV,			///< Comparison voltage.
	C17_SVDC_1500MV,			///< Comparison voltage.
	C17_SVDC_1550MV,			///< Comparison voltage.
	C17_SVDC_1600MV,			///< Comparison voltage.
	C17_SVDC_1650MV,			///< Comparison voltage.
	C17_SVDC_1700MV,			///< Comparison voltage.
	C17_SVDC_1800MV,			///< Comparison voltage.
	C17_SVDC_1900MV,			///< Comparison voltage.
	C17_SVDC_2000MV,			///< Comparison voltage.
	C17_SVDC_2100MV,			///< Comparison voltage.
	C17_SVDC_2200MV,			///< Comparison voltage.
	C17_SVDC_2300MV,			///< Comparison voltage.
	C17_SVDC_2400MV,			///< Comparison voltage.
	C17_SVDC_2500MV,			///< Comparison voltage.
	C17_SVDC_2600MV,			///< Comparison voltage.
	C17_SVDC_2700MV,			///< Comparison voltage.
	C17_SVDC_2800MV,			///< Comparison voltage.
	C17_SVDC_2900MV,			///< Comparison voltage.
	C17_SVDC_3000MV,			///< Comparison voltage.
	C17_SVDC_3100MV,			///< Comparison voltage.
	C17_SVDC_3200MV,			///< Comparison voltage.
	C17_SVDC_3300MV,			///< Comparison voltage.
	C17_SVDC_3400MV,			///< Comparison voltage.
	C17_SVDC_3500MV,			///< Comparison voltage.
	C17_SVDC_3600MV,			///< Comparison voltage.
	C17_SVDC_3700MV				///< Comparison voltage.
};

/* --- external declaration --- */
extern void c17intSvd(void)	__attribute__ ((interrupt_handler));

extern void c17initSvd(void);
extern void c17startSvd(void);
extern void c17stopSvd(void);
extern void c17setSvdComparisonVoltage(unsigned short svdVolt);
extern unsigned short c17getSvdVoltageDetection(void);

#endif	// C17_SVD_H__

