/************************************************************************//**
 *
 * @file    c17_tsrvr.c
 *
 * @brief   This is TSRVR driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_tsrvr.h"
#include "c17_init_config.h"

#if defined(C17_PE_TSRVR)
/* --- global variable --- */

/* --- static variable --- */

/************************************************************************//**
 * @brief   start temperature measurement.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableTempMeas(void)
{
	TSRVRNTCTL_REG	tsrvrnctl_set;

	/* TSRVR0TCTL: TSRVR Ch.0 Temperature Sensor Control Register */
	{
		tsrvrnctl_set.reg = TSRVR0TCTL;
		tsrvrnctl_set.bit.TEMPEN = 1;		// Enable temperature sensor output
		TSRVR0TCTL = tsrvrnctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   stopTempMeas function.
 *
 * @param   none.
 *
 * @retval  none
 ***************************************************************************/
void c17disableTempMeas(void)
{
	TSRVRNTCTL_REG	tsrvrnctl_set;

	/* TSRVR0TCTL: TSRVR Ch.0 Temperature Sensor Control Register */
	{
		tsrvrnctl_set.reg = TSRVR0TCTL;
		tsrvrnctl_set.bit.TEMPEN = 0;		// Disable temperature sensor output
		TSRVR0TCTL = tsrvrnctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   set reference voltage generator output voltage.
 *
 * @param   vref_md
 *  @li @c  C17_TSRVR_VREF_HIZ      Output Voltage = Hi_Z
 *  @li @c  C17_TSRVR_VREF_VDD      Output Voltage = VDD
 *  @li @c  C17_TSRVR_VREF_2_0V     Output Voltage = 2.0V
 *  @li @c  C17_TSRVR_VREF_2_5V     Output Voltage = 2.5V
 *
 * @retval  none
 ***************************************************************************/
void c17setVrefMode(unsigned short vref_md)
{
	TSRVRNVCTL_REG	tsrvrnvctl_set;

	/* TSRVR0TCTL: TSRVR Ch.0 Temperature Sensor Control Register */
	{
		tsrvrnvctl_set.reg = TSRVR0VCTL;
		tsrvrnvctl_set.bit.VREFAMD = vref_md;		/// -VREFAMD: reference voltage generator output voltage.
		TSRVR0VCTL = tsrvrnvctl_set.reg;
	}
}
#endif

