/************************************************************************//**
 *
 * @file    c17_uart.c
 *
 * @brief   This is UART driver program file.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2017. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_uart.h"
#include "c17_t16.h"
#include "c17_clg.h"
#include "c17_init_config.h"

#if defined(C17_PE_UART_MASTER) || defined(C17_PE_UART_SLAVE)

/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#if defined(C17_MCUSEL_M01)
C17_INTERRUPT_HANDLER(_vector9_handler, c17intUartCh0);		// UART Ch.0 interrupt
#endif
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589)
C17_INTERRUPT_HANDLER(_vector10_handler, c17intUartCh0);	// UART Ch.0 interrupt
#endif
#endif /* CRT0_LINKED */

/* --- global variable --- */
extern volatile unsigned int c17t16Ch0IntCount;				///< Interruption counter.

/* --- macro definition --- */
#define CALIBRATION_CYCLE	(255)							/// 255 cycle
#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W22)
#define CH_MAX_COUNT	1
#endif
#if defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18)
#define CH_MAX_COUNT	2
#endif
#if defined(C17_MCUSEL_589)
#define CH_MAX_COUNT	3
#endif

/* --- static const variable --- */
#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W22)
volatile UANCLK_REG* const uanclk_table[CH_MAX_COUNT] = {(UANCLK_REG *)&UA0CLK};
volatile UANMOD_REG* const uanmod_table[CH_MAX_COUNT] = {(UANMOD_REG *)&UA0MOD};
volatile UANBR_REG* const uanbr_table[CH_MAX_COUNT] = {(UANBR_REG *)&UA0BR};
volatile UANCTL_REG* const uanctl_table[CH_MAX_COUNT] = {(UANCTL_REG *)&UA0CTL};
volatile UANTXD_REG* const uantxd_table[CH_MAX_COUNT] = {(UANTXD_REG *)&UA0TXD};
volatile UANRXD_REG* const uanrxd_table[CH_MAX_COUNT] = {(UANRXD_REG *)&UA0RXD};
volatile UANINTF_REG* const uanintf_table[CH_MAX_COUNT] = {(UANINTF_REG *)&UA0INTF};
volatile UANINTE_REG* const uaninte_table[CH_MAX_COUNT] = {(UANINTE_REG *)&UA0INTE};
#endif
#if defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18)
volatile UANCLK_REG* const uanclk_table[CH_MAX_COUNT] = {(UANCLK_REG *)&UA0CLK,(UANCLK_REG *)&UA1CLK};
volatile UANMOD_REG* const uanmod_table[CH_MAX_COUNT] = {(UANMOD_REG *)&UA0MOD,(UANMOD_REG *)&UA1MOD};
volatile UANBR_REG* const uanbr_table[CH_MAX_COUNT] = {(UANBR_REG *)&UA0BR,(UANBR_REG *)&UA1BR};
volatile UANCTL_REG* const uanctl_table[CH_MAX_COUNT] = {(UANCTL_REG *)&UA0CTL,(UANCTL_REG *)&UA1CTL};
volatile UANTXD_REG* const uantxd_table[CH_MAX_COUNT] = {(UANTXD_REG *)&UA0TXD,(UANTXD_REG *)&UA1TXD};
volatile UANRXD_REG* const uanrxd_table[CH_MAX_COUNT] = {(UANRXD_REG *)&UA0RXD,(UANRXD_REG *)&UA1RXD};
volatile UANINTF_REG* const uanintf_table[CH_MAX_COUNT] = {(UANINTF_REG *)&UA0INTF,(UANINTF_REG *)&UA1INTF};
volatile UANINTE_REG* const uaninte_table[CH_MAX_COUNT] = {(UANINTE_REG *)&UA0INTE,(UANINTE_REG *)&UA1INTE};
#endif
#if defined(C17_MCUSEL_589)
volatile UANCLK_REG* const uanclk_table[CH_MAX_COUNT] = {(UANCLK_REG *)&UA0CLK,(UANCLK_REG *)&UA1CLK,(UANCLK_REG *)&UA2CLK};
volatile UANMOD_REG* const uanmod_table[CH_MAX_COUNT] = {(UANMOD_REG *)&UA0MOD,(UANMOD_REG *)&UA1MOD,(UANMOD_REG *)&UA2MOD};
volatile UANBR_REG* const uanbr_table[CH_MAX_COUNT] = {(UANBR_REG *)&UA0BR,(UANBR_REG *)&UA1BR,(UANBR_REG *)&UA2BR};
volatile UANCTL_REG* const uanctl_table[CH_MAX_COUNT] = {(UANCTL_REG *)&UA0CTL,(UANCTL_REG *)&UA1CTL,(UANCTL_REG *)&UA2CTL};
volatile UANTXD_REG* const uantxd_table[CH_MAX_COUNT] = {(UANTXD_REG *)&UA0TXD,(UANTXD_REG *)&UA1TXD,(UANTXD_REG *)&UA2TXD};
volatile UANRXD_REG* const uanrxd_table[CH_MAX_COUNT] = {(UANRXD_REG *)&UA0RXD,(UANRXD_REG *)&UA1RXD,(UANRXD_REG *)&UA2RXD};
volatile UANINTF_REG* const uanintf_table[CH_MAX_COUNT] = {(UANINTF_REG *)&UA0INTF,(UANINTF_REG *)&UA1INTF,(UANINTF_REG *)&UA2INTF};
volatile UANINTE_REG* const uaninte_table[CH_MAX_COUNT] = {(UANINTE_REG *)&UA0INTE,(UANINTE_REG *)&UA1INTE,(UANINTE_REG *)&UA2INTE};
#endif

/* --- static variable --- */
#if IOSC_CALIBRATION_ENABLE
static const unsigned short t16_div_table[16] = {1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768};		/* T16 clock division table */
#else
static const unsigned char uart_div_table[4] = {1,2,4,8};		/* UART clock division table */
#endif

typedef struct st_uart_buf {
	unsigned short size;
	unsigned short error;
    char *buf;
} ST_UART_S;

static ST_UART_S	uart_rcvbuff,uart_sendbuff;


#if IOSC_CALIBRATION_ENABLE
static int baudrateCalibration(unsigned char *uartBRT, unsigned char *uartFMD, unsigned long uartBps);
#else
static int baudrateSet(unsigned char *uartBRT, unsigned char *uartFMD, unsigned long uartBps);
#endif

/************************************************************************//**
 * @brief   initialize UART.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int  c17initUart(void)
{
	int result;
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589)
	P0UPMUX0_REG	p0upmux0_set;
	P0UPMUX1_REG	p0upmux1_set;
#endif
	UANCLK_REG		ua0clk_set;
	UANMOD_REG		ua0mod_set;
	UANBR_REG		ua0br_set;
	UANINTE_REG		ua0inte_set;
	UANINTF_REG		ua0intf_set;

	/// Stop UART.
	c17stopUart();

#if defined(C17_MCUSEL_M01)
	/// Initialize the UART input/output function to the ports.
	/*
	 *  P01 : USIN
	 *  P02 : USOUT
	 */
    /**
     * @todo change PORT
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P02SEL = 1;		/// - P02 : Enable peripheral I/O function.
		p0modsel_set.bit.P01SEL = 1;		/// - P01 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;

	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P02MUX = 1;		/// - P02 : USIN.
		p0fncsel_set.bit.P01MUX = 1;		/// - P01 : USOUT.
		P0FNCSEL = p0fncsel_set.reg;
	}
#endif

#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589)
	/// Initialize the UART input/output function to the ports.
	/*
	 *  P01 : USIN
	 *  P02 : USOUT
	 */
    /**
     * @todo change PORT
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P02SEL = 1;		/// - P02 : Enable peripheral I/O function.
		p0modsel_set.bit.P01SEL = 1;		/// - P01 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;

	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P02MUX = 1;		/// - P02 : UPMUX.
		p0fncsel_set.bit.P01MUX = 1;		/// - P01 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}
	/// Assign the UART input/output function to the ports.
	/* P0UPMUX0:P00-01 Universal Port Multiplexer Setting Register */
	{
		p0upmux0_set.reg = P0UPMUX0;
		p0upmux0_set.bit.P01PPFNC = 1;		/// - P01 : USIN.
		p0upmux0_set.bit.P01PERICH = 0;		/// - P01 : Ch.0.
		p0upmux0_set.bit.P01PERISEL	= 3;	/// - P01 : UART.
		P0UPMUX0 = p0upmux0_set.reg;
	}
	/* P0UPMUX1:P01-02 Universal Port Multiplexer Setting Register */
	{
		p0upmux1_set.reg = P0UPMUX1;
		p0upmux1_set.bit.P02PPFNC = 2;		/// - P02 : USOUT.
		p0upmux1_set.bit.P02PERICH = 0;		/// - P02 : Ch.0.
		p0upmux1_set.bit.P02PERISEL	= 3;	/// - P02 : UART.
		P0UPMUX1 = p0upmux1_set.reg;
	}
#endif

	/// Configure operating clock.
	/* UA0CLK: UART Ch.0 Clock Control Register */
	{
		ua0clk_set.reg = 0;
		ua0clk_set.bit.DBRUN = 1;									/// - Clock supplied in DEBUG mode.
		ua0clk_set.bit.CLKDIV = (UARTCH0_INIT & 0x30)>>4;			/// - Select the division ratio.
		ua0clk_set.bit.CLKSRC = (UARTCH0_INIT & 0xC0)>>6;			/// - Select the clock source of the UART.
		uanclk_table[0]->reg = ua0clk_set.reg;
	}

	/// Configure the following UA0MOD register bits.
	/* UA0MOD:UART Ch.0 Mode Register */
	{
	    /**
	     * @todo Change UA0MOD register.
	     */
		ua0mod_set.reg = 0;
		ua0mod_set.bit.INVIRRX = 0;									/// - Set USIN0 inverting(0=Disable input inverting / 1=Enable input inverting)
		ua0mod_set.bit.INVIRTX = 0;									/// - Set USOUT0 inverting(0=Disable output inverting / 1=Enable output inverting)
		ua0mod_set.bit.PUEN = 1;									/// - Set USIN0 pin(0=Disable pull-up / 1=Enable pull-up).
		ua0mod_set.bit.OUTMD = 0;									/// - Set USOUT0 pin(0=Push-pull output / 1=open-drain output).
		ua0mod_set.bit.IRMD = 0;									/// - IrDA interface(0=Disable / 1=Enable).
		ua0mod_set.bit.CHLN = (UARTCH0_INIT & 0x01);				/// - Data length(0=7-bit  / 1=8-bit).
		ua0mod_set.bit.PREN = ((UARTCH0_INIT & 0x0C)==0x00) ? 0:1;	/// - Parity function(0=Disable / 1=Enable).
		ua0mod_set.bit.PRMD = ((UARTCH0_INIT & 0x0C)==0x04) ? 1:0;	/// - Even parity selection(0=Unused / 1=Used).
		ua0mod_set.bit.STPB = (UARTCH0_INIT & 0x02)>>1;				/// - Stop bit length(0=1-bit  / 1=2-bit).
		uanmod_table[0]->reg = ua0mod_set.reg;
	}

	/* UA0BR:UART Ch.0 Baud-Rate Register */
	{
		ua0br_set.reg = 0;
		unsigned char BRT;
		unsigned char FMD;

#if IOSC_CALIBRATION_ENABLE
	    /**
	     * if UART Clock source = IOSC
	     * Execution of frequency AutoTriming at T16 ch.0(OSC1) & T16 ch1(IOSC)
	     *
	     */
		result = baudrateCalibration(&BRT, &FMD, UART_BSP);
		ua0br_set.bit.BRT = BRT;
		ua0br_set.bit.FMD = FMD;
#else
		/// Set transfer rate(bps = CLK_UART / {(BRT + 1) * 16 + FMD}).
		/// - S1C17M01       : 9600 bps
		/// - S1C17W22/W23   : 9600 bps
		/// - S1C17W03/W04   : 9600 bps
		/// - S1C17W14/W16   : 9600 bps
		/// - S1C17W15       : 9600 bps
		/// - S1C17W18       : 9600 bps
		/// - S1C17589       : 9600 bps

		result = baudrateSet(&BRT, &FMD, UART_BSP);
		ua0br_set.bit.BRT = BRT;
		ua0br_set.bit.FMD = FMD;
#endif
		uanbr_table[0]->reg = ua0br_set.reg;
	}

	/// Clear interrupt flag.
	/*  UA0INTF: UART Ch.0 Status and Interrupt Flag Register */
	{
		ua0intf_set.reg = 0;
		ua0intf_set.bit.TENDIF = 1;
		ua0intf_set.bit.FEIF = 1;
		ua0intf_set.bit.PEIF = 1;
		ua0intf_set.bit.OEIF = 1;
		uanintf_table[0]->reg = ua0intf_set.reg;
	}

	/* UA0INTE:UART Ch.0 Interrupt Enable Register */
	{
		ua0inte_set.reg = 0;
		ua0inte_set.bit.TENDIE = 1;
		ua0inte_set.bit.FEIE = 1;
		ua0inte_set.bit.PEIE = 1;
		ua0inte_set.bit.OEIE = 1;
		ua0inte_set.bit.RB2FIE = 0;
		ua0inte_set.bit.RB1FIE = 1;
		ua0inte_set.bit.TBEIE = 0;
		uaninte_table[0]->reg = ua0inte_set.reg;
	}

	/// Set interrupt level.
	ILVUART_0 = INTERRUPT_LEVEL_UART;	/// - Set interrupt level.


	return result;
}


/************************************************************************//**
 * @brief   Set UART enable.
 *
 * @param   none
 *
 * @retval none
 ***************************************************************************/
void c17startUart(void)
{
	UANCTL_REG		ua0ctl_set;
	UANINTF_REG		ua0intf_set;

	/// This variable is for dummy reading.
	/// Warning comes out, but there is no problem.
	int dummy_rd;

	/// clear flag
	dummy_rd = uanrxd_table[0]->reg;		// RXD dummy read

	/// Clear interrupt flag.
	/*  UA0INTF: UART Ch.0 Status and Interrupt Flag Register */
	{
		ua0intf_set.reg = 0;
		ua0intf_set.bit.TENDIF = 1;
		ua0intf_set.bit.FEIF = 1;
		ua0intf_set.bit.PEIF = 1;
		ua0intf_set.bit.OEIF = 1;
		uanintf_table[0]->reg = ua0intf_set.reg;
	}

	/// Execute software reset.
	/// Enable UART Ch.0 operations.
	/* UA0CTL:UART Ch.0 Control Register */
	{
		ua0ctl_set.bit.SFTRST = 1;
		ua0ctl_set.bit.MODEN = 0;
		uanctl_table[0]->reg = ua0ctl_set.reg;

		ua0ctl_set.bit.SFTRST = 0;
		ua0ctl_set.bit.MODEN = 1;
		uanctl_table[0]->reg = ua0ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set UART disable.
 *
 * @param   none
 *
 * @retval none
 ***************************************************************************/
void c17stopUart(void)
{
	UANCTL_REG		ua0ctl_set;

	/// Disable UART Ch.0 operations.
	/// Execute software reset.
	/* UA0CTL:UART Ch.0 Control Register */
	{
		ua0ctl_set.bit.SFTRST = 0;
		ua0ctl_set.bit.MODEN = 0;
		uanctl_table[0]->reg = ua0ctl_set.reg;

		ua0ctl_set.bit.SFTRST = 1;
		ua0ctl_set.bit.MODEN = 0;
		uanctl_table[0]->reg = ua0ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   UART Interrupt enable
 *
 * @param   uartRcvUnit
 *  @li @c  C17_UART_INT_RB1F   1Byte full interrupt enable.
 *  @li @c  C17_UART_INT_RB2F   2Byte full interrupt enable.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntUart(unsigned short uartRcvUnit)
{
	UANINTF_REG		ua0intf_set;
	UANINTE_REG		ua0inte_set;

	/// Clear interrupt flag.
	/*  UA0INTF: UART Ch.0 Status and Interrupt Flag Register */
	{
		ua0intf_set.reg = 0;
		ua0intf_set.bit.TENDIF = 1;
		ua0intf_set.bit.FEIF = 1;
		ua0intf_set.bit.PEIF = 1;
		ua0intf_set.bit.OEIF = 1;
		uanintf_table[0]->reg = ua0intf_set.reg;
	}

	/* UA0INTE:UART Ch.0 Interrupt Enable Register */
	{
	    /**
	     * @todo Change enable flag.
	     */
		ua0inte_set.reg = 0;
		ua0inte_set.bit.TENDIE = 1;
		ua0inte_set.bit.FEIE = 1;
		ua0inte_set.bit.PEIE = 1;
		ua0inte_set.bit.OEIE = 1;
		if(uartRcvUnit == C17_UART_RCV_1BYTE)
		{
			ua0inte_set.bit.RB2FIE = 0;
			ua0inte_set.bit.RB1FIE = 1;
		}
		else
		{
			ua0inte_set.bit.RB2FIE = 1;
			ua0inte_set.bit.RB1FIE = 0;
		}
		ua0inte_set.bit.TBEIE = 0;
		uaninte_table[0]->reg = ua0inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   UART Interrupt disable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntUart(void)
{
	UANINTE_REG		ua0inte_set;

	/* UA0INTE:UART Ch.0 Interrupt Enable Register */
	{
		ua0inte_set.reg = 0;
		ua0inte_set.bit.TENDIE = 0;
		ua0inte_set.bit.FEIE = 0;
		ua0inte_set.bit.PEIE = 0;
		ua0inte_set.bit.OEIE = 0;
		ua0inte_set.bit.RB2FIE = 0;
		ua0inte_set.bit.RB1FIE = 0;
		ua0inte_set.bit.TBEIE = 0;
		uaninte_table[0]->reg = ua0inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   First, It checks in underflow interruption.
 *
 *          Next, interruption flag is cleared.
 *
 * @param   none
 *
 * @retval none
 ***************************************************************************/
void c17intUartCh0(void)
{
	UANINTF_REG	ua0intf_set;
	UANINTE_REG	ua0inte_set;

	/* Capture interrupt flag */
	ua0intf_set.reg = UA0INTF;
	/* Clear Interrupt flag. */
	UA0INTF = ua0intf_set.reg;

	ua0inte_set.reg = uaninte_table[0]->reg;
	/// Check interrupt cause occurrence status.
	if (ua0intf_set.bit.TBEIF == 1) {
	   /**
		* @todo add interrupt Transmit buffer empty interrupt
		*/
	}
	if (ua0intf_set.bit.RB1FIF == 1) {

	   /**
		* @todo add interrupt Receive buffer one byte full interrupt
		*/
		if(ua0inte_set.bit.RB1FIE == 1)
		{
			/// 1Byte Receive data.
			if(uart_rcvbuff.size !=0)
			{
				*uart_rcvbuff.buf++ = uanrxd_table[0]->reg;
				uart_rcvbuff.size--;
			}
		}
	}
	if (ua0intf_set.bit.RB2FIF == 1) {
	   /**
		* @todo add interrupt  Receive buffer two bytes full interrupt
		*/
		/// 2Byte Receive data.
		if(uart_rcvbuff.size !=0)
		{
			*uart_rcvbuff.buf++ = uanrxd_table[0]->reg;
			uart_rcvbuff.size--;
		}
		if(uart_rcvbuff.size !=0)
		{
			*uart_rcvbuff.buf++ = uanrxd_table[0]->reg;
			uart_rcvbuff.size--;
		}
	}
	if (ua0intf_set.bit.OEIF == 1) {
	   /**
		* @todo add interrupt Overrun error interrupt
		*/
		uart_rcvbuff.error = C17_UART_ERROR_OVERFLOW;
	}
	if (ua0intf_set.bit.PEIF == 1) {
	   /**
		* @todo add interrupt Parity error interrupt
		*/
		uart_rcvbuff.error = C17_UART_ERROR_PARITY;
	}
	if (ua0intf_set.bit.FEIF == 1) {
	   /**
		* @todo add interrupt Framing error interrupt
		*/
		uart_rcvbuff.error = C17_UART_ERROR_FRAMING;
	}
	if (ua0intf_set.bit.TENDIF == 1) {
	   /**
		* @todo add interrupt End-of-transmission interrupt
		*/
		/// Send data.
		if(uart_sendbuff.size != 0)
		{
			uantxd_table[0]->reg = *uart_sendbuff.buf;
			uart_sendbuff.buf++;
			uart_sendbuff.size--;
		}
	}
}


/************************************************************************//**
 * @brief   UART receive data read.
 *
 * @param   uartData   Pointer of receive data.
 * @param   uartSize   Receive Size
 * @param   uartSize   Receive Unit(C17_UART_INT_RB1F or C17_UART_INT_RB2F)
 *
 * @retval  none
 ***************************************************************************/
void c17rcvUartaData(char *uartData, unsigned short uartSize, unsigned short uartRcvUnit)
{
	uart_rcvbuff.buf = uartData;
	uart_rcvbuff.size = uartSize;
	uart_rcvbuff.error = 0;

	c17enableIntUart(uartRcvUnit);

}


/****************************************************************************
 * receiveUartData Polling  function.
 *
 * @brief	UART receive data read.
 *
 * @param	uartData	Pointer of transmit data.
 * @param	uartSize	Transmit data size.
 *
 * @retval	EXIT_SUCCESS	Success.
 * @retval  Other           Failed.
 ***************************************************************************/
int c17rcvUartDataPolingByte(char *uartData, unsigned short uartSize)
{
	unsigned short i;
	UANINTF_REG	ua0intf_set;
	/// Receive data.
	for (i = 0; i < uartSize; i++) {

		/// Check receive buffer one byte full interrupt.
		while (uanintf_table[0]->bit.RB1FIF == 0) {
			/// Wait RB1FIF Interrupt  for the receive buff full.
			/**
			 * @todo Add time-out processing as needed.
			 */
		}

		/// Read receive data.
		*uartData++ = uanrxd_table[0]->reg;

		ua0intf_set.reg = uanintf_table[0]->reg;
		if(ua0intf_set.bit.OEIF == 1)
		{
			/// Overrun error
			return C17_UART_ERROR_OVERFLOW;
		}
		if(ua0intf_set.bit.PEIF == 1)
		{
			/// Parity error
			return C17_UART_ERROR_PARITY;
		}
		if(ua0intf_set.bit.FEIF == 1)
		{
			/// Framing error
			return C17_UART_ERROR_FRAMING;
		}
	}

	return EXIT_SUCCESS;
}


/****************************************************************************
 * receiveUartData Polling  function.
 *
 * @brief	UART receive data read.
 *
 * @param	uartData	Pointer of transmit data.
 * @param	uartSize	Transmit data size.
 *
 * @retval	EXIT_SUCCESS	Success.
 * @retval  Other           Failed.
 ***************************************************************************/
int c17rcvUartDataPolingShort(unsigned short *uartData, unsigned short uartSize)
{
	unsigned short i;
	unsigned short high_data;
	unsigned short low_data;
	UANINTF_REG	ua0intf_set;

	/// Receive data.
	for (i = 0 ;i<uartSize;i++) {

		/// Check receive buffer one byte full interrupt.
		while (uanintf_table[0]->bit.RB2FIF == 0) {
			/// Wait RB2FIF Interrupt  for the receive buff full.
			/**
			 * @todo Add time-out processing as needed.
			 */

		}

		/// Read receive data.
		low_data = uanrxd_table[0]->reg;
		high_data = uanrxd_table[0]->reg;

		*uartData++ = ((high_data<<8) + low_data);

		ua0intf_set.reg = uanintf_table[0]->reg;
		if(ua0intf_set.bit.OEIF == 1)
		{
			/// Overrun error
			return C17_UART_ERROR_OVERFLOW;
		}
		if(ua0intf_set.bit.PEIF == 1)
		{
			/// Parity error
			return C17_UART_ERROR_PARITY;
		}
		if(ua0intf_set.bit.FEIF == 1)
		{
			/// Framing error
			return C17_UART_ERROR_FRAMING;
		}
	}

	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief   UART transmit data write.
 *
 * @param   uartData    Pointer of transmit data.
 * @param   uartSize    Transmit data size.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17sendUartData(char *uartData, unsigned short uartSize)
{
	UANINTF_REG	ua0intf_set;

	if(uartSize == 0)
	{
		return EXIT_FAILURE;
	}

	/// Clear interrupt flag.
	/*  UA0INTF: UART Ch.0 Status and Interrupt Flag Register */
	{
		ua0intf_set.reg = 0;
		ua0intf_set.bit.TENDIF = 1;
		ua0intf_set.bit.FEIF = 1;
		ua0intf_set.bit.PEIF = 1;
		ua0intf_set.bit.OEIF = 1;
		uanintf_table[0]->reg = ua0intf_set.reg;
	}

	uart_sendbuff.buf = uartData;
	uart_sendbuff.size = uartSize;

	/// Send data.
	uantxd_table[0]->reg = *uart_sendbuff.buf;
	uart_sendbuff.buf++;
	uart_sendbuff.size--;

	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief   UART receive end wait.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  Other           Failed.
 ***************************************************************************/
int c17rcvUartDataWait(void)
{
	if(uart_rcvbuff.size == 0)
	{
		if(uart_rcvbuff.error == 0){
			return EXIT_SUCCESS;
		}
		else{
			return uart_rcvbuff.error;
		}
	}
	else
	{
		if(uart_rcvbuff.error == 0){
			return EXIT_FAILURE;
		}
		else{
			return uart_rcvbuff.error;
		}
	}
}


/************************************************************************//**
 * @brief   UART transmit end wait.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17sendUartDataWait(void)
{
	if(uart_sendbuff.size == 0)
	{
		return EXIT_SUCCESS;
	}
	else
	{
		return EXIT_FAILURE;
	}
}


/****************************************************************************
 * sendUartData Polling function.
 *
 * @brief	UART transmit data write.
 *
 * @param	uartData	Pointer of transmit data.
 * @param	uartSize	Transmit data size.
 *
 * @retval	none
 ***************************************************************************/
void c17sendUartDataPoling(char *uartData, unsigned short uartSize)
{
	unsigned short i;

	if(uartSize == 0)
	{
		return;
	}

	// Send data.
	for (i = 0 ; i < uartSize ; i++) {

		/// Check Transmit buffer empty interrupt.
		while (uanintf_table[0]->bit.TBEIF == 0) {
			/// Wait TBEIF Interrupt  for the send buff empty.
			/**
			 * @todo Add time-out processing as needed.
			 */
		}

		/// Send data.
		uantxd_table[0]->reg = *uartData++;
	}
}


#if IOSC_CALIBRATION_ENABLE
/************************************************************************//**
 * @brief   UART Calibration
 *
 * @param   uartBRT    Pointer of BRT data.
 * @param   uartFMD    Pointer of FMD data.
 * @param   uartBps    BPS.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int baudrateCalibration(unsigned char *uartBRT, unsigned char *uartFMD, unsigned long uartBps)
{
	unsigned long temp1,temp2,t16Cnt;
	unsigned short division;
	T16_NCLK_REG	t16_nclk_set;

	/**
	* @todo change start clock source function.
	*
	* If the UART clock source of the IOSC,
    * to adjust the baud rate using the other clock(OSC3 or OSC1)
	* example OSC1
	*/
	c17startClgOSC1();

#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)|| \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18)
   /**
	* @todo change Clock Source, Clock Division.
	*              Please set the division ratio so that the frequency is in the near of 32KHz.
	*              OSC1=32.768KHz, CLKDIV=1/1 [(CALIBRATION_CYCLE)/(32.768kHz/1) = 0.00778 sec.]
	*              if IOSC=700KHz, CLKDIV=1/1 [(65535)/(700KHz/1)= 0.0936 sec.]
	*/
	/// Initialize T16 CH.0,CH.1 timer.
	c17setT16(0, C17_T16_CLKSRC_OSC1, C17_T16_CLKDIV_1, C17_T16_MODE_ONESHOT);		/// For adjust the baud rate using.
	c17setT16(1, C17_T16_CLKSRC_IOSC, C17_T16_CLKDIV_1, C17_T16_MODE_REPEAT);		/// For adjust the baud rate using.
#endif
#if defined(C17_MCUSEL_589)
   /**
	* @todo change Clock Source, Clock Division.
	*              Please set the division ratio so that the frequency is in the near of 32KHz.
	*              OSC1=32.768KHz, CLKDIV=1/1 [(CALIBRATION_CYCLE)/(32.768kHz/1) = 0.00778 sec.]
	*              if IOSC=16MHz, CLKDIV=1/16 [(65535)/(16MHz/16)= 0.0655 sec.]
	*/
	/// Initialize T16 CH.0,CH.1 timer.
	c17setT16(0, C17_T16_CLKSRC_OSC1, C17_T16_CLKDIV_1, C17_T16_MODE_ONESHOT);		/// For adjust the baud rate using.
	c17setT16(1, C17_T16_CLKSRC_IOSC, C17_T16_CLKDIV_16, C17_T16_MODE_REPEAT);		/// For adjust the baud rate using.
#endif

	/// Initialize the interrupt.
	ILVT16_0 = INTERRUPT_LEVEL_T16_0;				/// - Set interrupt level.

	/// Set Reload Value T16 CH.0
	c17setCountT16(0,CALIBRATION_CYCLE - 1);

	/// T16 CH.0 Interrupt Enable
	c17enableIntT16(0);

	/// Set Reload Value T16 CH.1
	c17setCountT16(1, 0xFFFF);

	c17t16Ch0IntCount = 0;								///< Interruption counter.

	/// Start T16 CH.0,CH.1 timer.
	c17startT16(0);
	c17startT16(1);

	/// Get T16 interrupt count.
	while(c17t16Ch0IntCount == 0){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");
	}

	/// Stop T16 CH0,CH1 timer.
	c17stopT16(1);
	c17stopT16(0);

#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)|| \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589)
   /**
	* @todo change stop clock source function.
	* example OSC1
	*/
	c17stopClgOSC1();
#endif

	// T16 CH.1 counter read
	t16Cnt	= ~c17getCountT16(1);

	// T16 CH.0 clock division
	t16_nclk_set.reg = T16_0CLK;
	division = t16_nclk_set.bit.CLKDIV;

	temp1 = (t16Cnt * (IOSC_CALIBRATION_CLOCK / t16_div_table[division])) * 10;
	temp1 /= (uartBps * CALIBRATION_CYCLE) + 5;
	temp1 /= 10;

	t16_nclk_set.reg = T16_1CLK;
	division = t16_nclk_set.bit.CLKDIV;

	temp1 = temp1 * t16_div_table[division];

	temp2 = temp1 / 16 -1;
	*uartBRT = temp2;
	*uartFMD = temp1 % 16;

	if(temp2 > 0xFF)
	{
		//Overflow error
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}


#else
/************************************************************************//**
 * @brief   Set baud rate
 *
 * @param   uartBRT    Pointer of BRT data.
 * @param   uartFMD    Pointer of FMD data.
 * @param   uartBps    BPS.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int baudrateSet(unsigned char *uartBRT, unsigned char *uartFMD, unsigned long uartBps)
{
	unsigned long clock,temp1,temp2;
	UANCLK_REG		ua0clk_set;

	ua0clk_set.reg = uanclk_table[0]->reg;
#if defined(C17_MCUSEL_M01)
	if(ua0clk_set.bit.CLKSRC == C17_UART_CLKSRC_IOSC)
	{
		clock = UART_CLOCK / uart_div_table[ua0clk_set.bit.CLKDIV];
	}
#endif
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589)
	if((ua0clk_set.bit.CLKSRC == C17_UART_CLKSRC_IOSC) || (ua0clk_set.bit.CLKSRC == C17_UART_CLKSRC_OSC3))
	{
		clock = UART_CLOCK / uart_div_table[ua0clk_set.bit.CLKDIV];
	}
#endif
	else
	{
		clock = UART_CLOCK;
	}

	temp1 = clock *10 / uartBps + 5;

	temp1 /= 10;
	temp2 =  temp1 / 16-1;

	*uartBRT = temp2;
	*uartFMD = temp1 % 16;

	if(temp2 > 0xFF)
	{
		//Overflow error
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
#endif

#endif

