/************************************************************************//**
 *
 * @file    c17_wdt.c
 *
 * @brief   This is Watch Dog Timer program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "c17_misc.h"
#include "c17_wdt.h"
#include "c17_init_config.h"

#if defined(C17_PE_WDT)

/************************************************************************//**
 * @brief   Initialize watch dog Timer.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS        Success.
 * @retval  EXIT_FAILURE        Failed.
 ***************************************************************************/
void c17initWdt(void)
{
	WDTCLK_REG	wdtclk_set;
	WDTCTL_REG	wdtctl_set;

	/// Stop watch dog Timer.
	c17stopWdt();

	/// Disable write-protect.
	c17protectMiscOff();

	/// Set clock source for watch dog Timer.
	/* WDTCLK:WDT Clock Control Register */
	{
		 /**
		 * @todo Change clock source & Clock division ratio selection
		 */
		wdtclk_set.bit.DBRUN = 1;		/// - Clock supplied in DEBUG mode(0:Disable 1:Enable).
		wdtclk_set.bit.CLKDIV = 0;		/// - Set division ratio for clock source(1/128).
		wdtclk_set.bit.CLKSRC = 1;		/// - Set clock source(OSC1).
		WDTCLK = wdtclk_set.reg;
	}

	/// Reset watch dog Timer count.
	/* WDTCTL:WDT Control Register */
	{
		wdtctl_set.bit.WDTCNTRST = 1;	/// - Reset
		wdtctl_set.bit.WDTRUN = 0xa;	/// - Stop
		WDTCTL = wdtctl_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();

}


/************************************************************************//**
 * @brief   Start watchdog Timer.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startWdt(void)
{
	WDTCTL_REG	wdtctl_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/// Start watch dog Timer.
	/// Reset watch dog Timer count.
	/* WDTCTL:WDT Control Register */
	{
		wdtctl_set.reg = WDTCTL;
		wdtctl_set.bit.WDTCNTRST = 1;	/// - Reset
		wdtctl_set.bit.WDTRUN = 0x0;	/// - Start
		WDTCTL = wdtctl_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();
}


/************************************************************************//**
 * @brief   Stop watch dog Timer.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopWdt(void)
{
	WDTCTL_REG	wdtctl_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/// Stop watch dog Timer.
	/// Reset watch dog Timer count.
	/* WDTCTL:WDT Control Register */
	{
		wdtctl_set.reg = WDTCTL;
		wdtctl_set.bit.WDTCNTRST = 1;	/// - Reset
		wdtctl_set.bit.WDTRUN = 0xa;	/// - Stop
		WDTCTL = wdtctl_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();
}
#endif

