/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is REMC2 sample program for S1C17W18 demonstration.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @main page   S1C17W18 REMC2
 *
 * @section     intro Introduction
 * It is a sample code for REMC2.
 *
 * @subsection  step0   Interrupt every 1sec and start REMC2.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17w18_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_remc2.h"
#include "c17_init_config.h"

#ifdef C17_PE_REMC


/* --- global variable --- */


/* --- constant --- */

/// TV POWER on/off code
// NEC format
const unsigned char DataNEC[4] = {0x18,~0x18,0x10,~0x10};
const unsigned char DataSANYO[4] = {0x0C,~0x0C,0x00,~0x00};
const unsigned char DataTOSHIBA[4] = {0x02,~0x02,0x48,~0x48};
const unsigned char DataHITACHI[4] = {0x0A,0xF5,0xE8,0x17};
const unsigned char DataFUJITSU[4] = {0xA8,0x57,0x48,0xB7};
const unsigned char DataPIONEER[4] = {0x55,~0x55,0x38,~0x38};

// AEHA format
const unsigned char DataPANASONIC[6] = {0x40,0x04,0x01,0x00,0xBC,0xBD};
const unsigned char DataSHARP[6] = {0x55,0x5A,0xF1,0x48,0x68,0x8B};

// Sony format
const unsigned char DataSONY[2] = {0xA9,0x00};


/* --- function declaration --- */
int main_remc2(void);


/****************************************************************************
 * main function.
 *
 * @brief   1, Initialize REMC2.
 *
 *          2, Send NEC format data.
 *
 *          3, Send AEHA format data.
 *
 *          4, Send SONY format data.
 *
 *          5, Stop REMC2.
 *
 * @return 	EXIT_SUCCESS	Success.
 ***************************************************************************/
int main_remc2(void)
{
	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();
	c17stopClgOSC3();

	/// start OSC3(internal oscillator)
	c17startClgIntOSC3(C17_CLG_OSC3_4MHZ);

	/// the system clock set in OSC3.
	c17switchClgSystemClockOSC3();
	c17stopClgIOSC();

	/// Initialize REMC.
	c17initRemc();

	// NEC format
	c17setFormatRemc(C17_REMC_FORMAT_NEC);
	c17sendRemc(DataNEC, sizeof(DataNEC));
	while(c17sendRemcDataWait() == EXIT_FAILURE)
	{
		asm("halt");
		asm("nop");
	}
	c17stopRemc();

	// AEHA format
	c17setFormatRemc(C17_REMC_FORMAT_AEHA);
	c17sendRemc(DataPANASONIC, sizeof(DataPANASONIC));
	while(c17sendRemcDataWait() == EXIT_FAILURE)
	{
		asm("halt");
		asm("nop");
	}
	c17stopRemc();

	// SONY format
	c17setFormatRemc(C17_REMC_FORMAT_SONY);
	c17sendRemc(DataSONY, sizeof(DataSONY));
	while(c17sendRemcDataWait() == EXIT_FAILURE)
	{
		asm("halt");
		asm("nop");
	}
	c17stopRemc();


	c17disableRemc();

	return EXIT_SUCCESS;
}
#endif

